; #########################################################################

; This small demo came about from a discussion with Ernie Murphy on how
; to make the split WORD size parameters on the stack into WORD values
; directly without having to convert them from a DWORD parameter as they
; are passed to a message handling proc such as a WndProc. This technique
; is both simpler and more efficient that converting the values.

; #########################################################################

      .386
      .model flat, stdcall  ; 32 bit memory model
      option casemap :none  ; case sensitive

      include Ernie.inc     ; local includes for this file

    ; ---------------------------------------------------------
    ; These 4 equates demonstrate a simple way of splitting
    ; the wParam and lParam passed to a WndProc into their
    ; WORD size components. They take the WORD value directly
    ; off the stack without any conversions.
    ; ---------------------------------------------------------
      wpLow  equ <WORD PTR [ebp + 16]>
      wpHigh equ <WORD PTR [ebp + 18]>
      lpLow  equ <WORD PTR [ebp + 20]>
      lpHigh equ <WORD PTR [ebp + 22]>

; #########################################################################

.code

start:
      invoke GetModuleHandle, NULL
      mov hInstance, eax

      invoke GetCommandLine
      mov CommandLine, eax

      invoke InitCommonControls

      invoke WinMain,hInstance,NULL,CommandLine,SW_SHOWDEFAULT
      invoke ExitProcess,eax

; #########################################################################

WinMain proc hInst     :DWORD,
             hPrevInst :DWORD,
             CmdLine   :DWORD,
             CmdShow   :DWORD

      ;====================
      ; Put LOCALs on stack
      ;====================

      LOCAL wc   :WNDCLASSEX
      LOCAL msg  :MSG

      ;==================================================
      ; Fill WNDCLASSEX structure with required variables
      ;==================================================

      szText szClassName,"Project_Class"

      mov wc.cbSize,         sizeof WNDCLASSEX
      mov wc.style,          CS_HREDRAW or CS_VREDRAW \
                             or CS_BYTEALIGNWINDOW
      mov wc.lpfnWndProc,    offset WndProc
      mov wc.cbClsExtra,     NULL
      mov wc.cbWndExtra,     NULL
      m2m wc.hInstance,      hInst
      mov wc.hbrBackground,  NULL ; COLOR_BTNFACE+1
      mov wc.lpszMenuName,   NULL
      mov wc.lpszClassName,  offset szClassName
        invoke LoadIcon,hInst,500    ; icon ID
        mov hIcon, eax
      m2m wc.hIcon,          eax
      m2m wc.hIconSm,        eax
        invoke LoadCursor,NULL,IDC_ARROW
      mov wc.hCursor,        eax

      invoke RegisterClassEx, ADDR wc

      ;================================
      ; Centre window at following size
      ;================================

      Wwd EQU 500
      Wht EQU 350

      invoke GetSystemMetrics,SM_CXSCREEN

      sub eax, Wwd    ; subtract window width from screen width
      shr eax, 1      ; divide by 2

      mov ebx, eax

      invoke GetSystemMetrics,SM_CYSCREEN

      sub eax, Wht    ; subtract window height from screen height
      shr eax, 1      ; divide by 2

      invoke CreateWindowEx,WS_EX_LEFT,
                            ADDR szClassName,
                            ADDR szDisplayName,
                            WS_OVERLAPPEDWINDOW,
                            ebx,eax,Wwd,Wht,
                            NULL,NULL,
                            hInst,NULL
      mov   hWnd,eax

      invoke LoadMenu,hInst,600  ; menu ID
      invoke SetMenu,hWnd,eax

      invoke ShowWindow,hWnd,SW_SHOWNORMAL
      invoke UpdateWindow,hWnd

      ;===================================
      ; Loop until PostQuitMessage is sent
      ;===================================

    StartLoop:
      invoke GetMessage,ADDR msg,NULL,0,0
      cmp eax, 0
      je ExitLoop
      invoke TranslateMessage, ADDR msg
      invoke DispatchMessage,  ADDR msg
      jmp StartLoop
    ExitLoop:

      return msg.wParam

WinMain endp

; #########################################################################

WndProc proc hWin   :DWORD,
             uMsg   :DWORD,
             wParam :DWORD,
             lParam :DWORD

    LOCAL var    :DWORD
    LOCAL caW    :DWORD
    LOCAL caH    :DWORD
    LOCAL Rct    :RECT
    LOCAL hDC    :DWORD
    LOCAL Ps     :PAINTSTRUCT
    LOCAL tbab   :TBADDBITMAP
    LOCAL tbb    :TBBUTTON
    LOCAL buffer1[128]:BYTE  ; these are two spare buffers
    LOCAL buffer2[128]:BYTE  ; for text manipulation etc..

    .if uMsg == WM_COMMAND

    ;======== menu commands ========
        .if wParam == 1010
            invoke SendMessage,hWin,WM_SYSCOMMAND,SC_CLOSE,NULL
        .elseif wParam == 1900
            szText AboutMsg,"Prostart Pure Assembler Template",13,10,\
            "Copyright  MASM32 2000"
            invoke ShellAbout,hWin,ADDR szDisplayName,ADDR AboutMsg,hIcon
        .endif

    ; ************************************************************

    .elseif uMsg == WM_LBUTTONDOWN
        invoke SetCapture,hWin
	invoke GetWindowRect,hWin, ADDR Rct
	add Rct.left,4
	add Rct.top,42
	sub Rct.right,3
	sub Rct.bottom,23
	invoke SetRect,ADDR Rct,Rct.left,Rct.top,Rct.right,Rct.bottom
	invoke ClipCursor,ADDR Rct
	jmp lbutton

    .elseif uMsg == WM_LBUTTONUP
	invoke ClipCursor,NULL
        call ReleaseCapture
	return 0

    .elseif uMsg == WM_MOUSEMOVE

        .if wParam == MK_LBUTTON
        ; ----------------------------------------
        ; both parameters work in SIGNED dword
        ; range so they must use SIGN extension
        ; ----------------------------------------
lbutton:  movsx edx, lpLow              ; convert WORD to DWORD
	  mov hitpoint.x,edx
          invoke dwtoa,edx,ADDR buffer1
          invoke SendMessage,hStatus,SB_SETTEXT,0,ADDR buffer1

          movsx edx, lpHigh             ; convert WORD to DWORD
	  mov hitpoint.y,edx
          invoke dwtoa,edx,ADDR buffer2
          invoke SendMessage,hStatus,SB_SETTEXT,1,ADDR buffer2

	  mov MouseClick,TRUE
	  invoke InvalidateRect,hWnd,NULL,TRUE
        .endif

    ; ************************************************************

    .elseif uMsg == WM_CREATE
      ; *******************
      ; Function call here
      ; *******************
        invoke VerticalTile,hWin,2,50
        invoke Do_Status,hWin

    .elseif uMsg == WM_SIZE
        invoke MoveWindow,hStatus,0,0,0,0,TRUE

    .elseif uMsg == WM_PAINT
        invoke BeginPaint,hWin,ADDR Ps
          mov hDC, eax
	  .IF MouseClick
	      invoke SetBkMode,hDC,TRANSPARENT 
	      invoke lstrlen,ADDR szDisplayName
	      invoke TextOut,hDC,hitpoint.x,hitpoint.y,ADDR szDisplayName,eax
	      invoke SetBkMode,hDC,OPAQUE 
	  .ENDIF
          invoke Paint_Proc,hWin,hDC
        return 0

    .elseif uMsg == WM_CLOSE

    .elseif uMsg == WM_DESTROY
        invoke PostQuitMessage,NULL
        return 0 
    .endif

    invoke DefWindowProc,hWin,uMsg,wParam,lParam

    ret

WndProc endp

; #########################################################################

Paint_Proc proc hWin:DWORD, hDC:DWORD

    LOCAL btn_hi   :DWORD
    LOCAL btn_lo   :DWORD
    LOCAL Rct      :RECT

    invoke GetSysColor,COLOR_BTNHIGHLIGHT
    mov btn_hi, eax

    invoke GetSysColor,COLOR_BTNSHADOW
    mov btn_lo, eax

    return 0

Paint_Proc endp

; ########################################################################

end start
